/********************************** (C) COPYRIGHT  *******************************
 * File Name          : iap.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2020/12/16
 * Description        : IAP
*********************************************************************************
* Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
* Attention: This software (modified or not) and binary are used for 
* microcontroller manufactured by Nanjing Qinheng Microelectronics.
*******************************************************************************/
#include "iap.h"
#include "string.h"
#include "flash.h"
#include "core_riscv.h"
/******************************************************************************/

#if CH32V20_IAP_BOOT
#define FLASH_Base   0x08002800
#define USBD_DATA_SIZE               64
iapfun jump2app;
u32 Program_addr = FLASH_Base;
u32 Verity_addr = FLASH_Base;
u32 User_APP_Addr_offset = 0x5000;
u8 Verity_Star_flag = 0;
u8 Fast_Program_Buf[390];
u32 CodeLen = 0;
u8 End_Flag = 0;
u8 EP2_Rx_Buffer[USBD_DATA_SIZE];
#define  isp_cmd_t   ((isp_cmd  *)EP2_Rx_Buffer)

/*********************************************************************
 * @fn      RecData_Deal
 *
 * @brief   UART-USB deal date
 *
 * @return  ERR_ERROR - ERROR
 *          ERR_SCUESS - SCUESS
 *          ERR_End - End
 */
void log_data_hex(void const *buf, uint16_t len)
{
	uint16_t i;
	uint8_t* p = buf;

	if(0 == len) return;
    for(i=0; i<len; i++){
        if((i % 16) == 0){
            PRINT("\r\n");
        }
        PRINT("%02x ",p[i]);
    }
    PRINT("\r\n");
}
u8 RecData_Deal(void)
{
     u8 i, s, Lenth;

     Lenth = isp_cmd_t->Len;

     switch ( isp_cmd_t->Cmd) {
     case CMD_IAP_ERASE:
         FLASH_Unlock_Fast();
         s = ERR_SCUESS;
         break;

     case CMD_IAP_PROM:
         for (i = 0; i < Lenth; i++) {
             Fast_Program_Buf[CodeLen + i] = isp_cmd_t->data[i];
         }
         CodeLen += Lenth;
         if (CodeLen >= 256) {
            PRINT("Program_addr:0x%x\r\n",Program_addr);
             FLASH_Unlock_Fast();
             FLASH_ErasePage_Fast(Program_addr);
             CH32_IAP_Program(Program_addr, (u32*) Fast_Program_Buf);
             CodeLen -= 256;
             for (i = 0; i < CodeLen; i++) {
                 Fast_Program_Buf[i] = Fast_Program_Buf[256 + i];
             }

             Program_addr += 0x100;

         }
         s = ERR_SCUESS;
         break;

     case CMD_IAP_VERIFY:
         if (Verity_Star_flag == 0) {
             Verity_Star_flag = 1;

             for (i = 0; i < (256 - CodeLen); i++) {
                 Fast_Program_Buf[CodeLen + i] = 0xff;
             }

             FLASH_ErasePage_Fast(Program_addr);
             CH32_IAP_Program(Program_addr, (u32*) Fast_Program_Buf);
             CodeLen = 0;
         }

         s = ERR_SCUESS;
         for (i = 0; i < Lenth; i++) {
             if (isp_cmd_t->data[i] != *(u8*) (Verity_addr + i)) {
                 s = ERR_ERROR;
                 break;
             }
         }

         Verity_addr += Lenth;

         break;

     case CMD_IAP_END:
         Verity_Star_flag = 0;
         End_Flag = 1;
         PRINT("Flag:%d\r\n",End_Flag);
         Program_addr = FLASH_Base;
         Verity_addr = FLASH_Base;

         s = ERR_End;
         break;

     default:
         s = ERR_ERROR;
         break;
     }

     return s;
 }

/*********************************************************************
 * @fn      USART2_CFG
 *
 * @brief   baudrate:Uart2 baudrate
 *
 * @return  none
 */
uint8_t uart_dma_buf[DMA_BUFF_SIZE];
void iap_uart_dma_init(void)
{
	DMA_InitTypeDef DMA_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;
	
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);	

    DMA_DeInit(DMA1_Channel6);
	DMA_InitStructure.DMA_PeripheralBaseAddr = (u32)&USART2->DATAR;  //DMA外设基地址
	DMA_InitStructure.DMA_MemoryBaseAddr = (u32)uart_dma_buf;  //DMA内存基地址
	DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;  //
	DMA_InitStructure.DMA_BufferSize = DMA_BUFF_SIZE;  //DMA通道的DMA缓存的大小
	DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;  //外设地址寄存器不变
	DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;  //内存地址寄存器递增
	DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;  //数据宽度为8位
	DMA_InitStructure.DMA_MemoryDataSize =  DMA_MemoryDataSize_Byte;  //数据宽度为8位
	DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;  //
	DMA_InitStructure.DMA_Priority = DMA_Priority_Medium; //DMA通道 x拥有中优先级
	DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;  //DMA通道x没有设置为内存到内存传输
	DMA_Init(DMA1_Channel6, &DMA_InitStructure); 

    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);
	NVIC_InitStructure.NVIC_IRQChannel = DMA1_Channel6_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;  
	NVIC_Init(&NVIC_InitStructure);
	DMA_ClearITPendingBit(DMA1_IT_TC6);
	DMA_ITConfig(DMA1_Channel6,DMA_IT_TC,ENABLE);
	DMA_Cmd(DMA1_Channel6, ENABLE); 
    USART_DMACmd(USART2, USART_DMAReq_Rx, ENABLE);
}

void USART2_CFG(u32 baudrate)
{
    GPIO_InitTypeDef GPIO_InitStructure = {0};
    USART_InitTypeDef USART_InitStructure = {0};

    RCC_APB2PeriphClockCmd( RCC_APB2Periph_GPIOA, ENABLE);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2,ENABLE);
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    USART_InitStructure.USART_BaudRate = baudrate;
    USART_InitStructure.USART_WordLength = USART_WordLength_8b;
    USART_InitStructure.USART_StopBits = USART_StopBits_1;
    USART_InitStructure.USART_Parity = USART_Parity_No;
    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_InitStructure.USART_Mode = USART_Mode_Tx | USART_Mode_Rx;

    USART_Init(USART2, &USART_InitStructure);
    USART_Cmd(USART2, ENABLE);
    iap_uart_dma_init();

}

uint16_t dma_get_rx_lenth(void)
{
    return ((DMA_BUFF_SIZE - DMA_GetCurrDataCounter(DMA1_Channel6)));
}
/*********************************************************************
 * @fn      Uart2_SendMultiyData
 *
 * @brief   Deal device Endpoint 3 OUT.
 *
 * @param   l: Data length.
 *
 * @return  none
 */
void Uart2_SendMultiyData(u8* pbuf, u8 num)
{
    u8 i = 0;

    while(i<num)
    {
        while(USART_GetFlagStatus(USART2, USART_FLAG_TC) == RESET);
        USART_SendData(USART2, pbuf[i]);
        i++;
    }
}
/*********************************************************************
 * @fn      Uart2_SendMultiyData
 *
 * @brief   USART2 send date
 *
 * @param   pbuf - Packet to be sent
 *          num - the number of date
 *
 * @return  none
 */
void Uart2_SendData(u8 data)
{
    while(USART_GetFlagStatus(USART2, USART_FLAG_TC) == RESET);
    USART_SendData(USART2, data);
}

/*********************************************************************
 * @fn      Uart2_Rx
 *
 * @brief   Uart2 receive date
 *
 * @return  none
 */
u8 Uart2_Rx(void)
{
    while( USART_GetFlagStatus(USART2, USART_FLAG_RXNE) == RESET);
    return USART_ReceiveData( USART2);
}
u8 get_uart_command(uint8_t c)
{
	static u8* s_buf = isp_cmd_t;
	static u8 s_len=0;
	static u8 data_len=0;
	static u8 ready = 0;
    static u8 Data_add = 0;
	u8 ret = 0;
    // PRINT("0x%x\n\r",c);
    if(ready == 0){
        if(c == Uart_Sync_Head1){
            ready = 1;
            // PRINT("ready:%d  0x%x\n\r",ready,c);
        }
    }else if(ready == 1){
        if(c == Uart_Sync_Head2){
            ready = 2;
        }else{
            ready = 0;
        }
        // PRINT("ready:%d  0x%x\n\r",ready,c);
    }else if(ready == 2){
        s_buf[s_len] = c;
        Data_add += c;
        if(s_len >= 1){
            if(s_len == 1){
                data_len = s_buf[1];
                // PRINT("data_len:%d\n\r",data_len);
            }else{
                data_len--;
            }
            if(data_len == 0){
                ready = 3;
            }
        }
        s_len++;
    }else if(ready == 3){
        // if(Data_add == c){
            // PRINT("Data_add:%X\n\r",Data_add);
        // }else{
            // PRINT("ERR: Sum!!! %x, %x\n\r",Data_add, c);
        // }
        ret = 1;
        ready = 0;
        s_len = 0;
        Data_add = 0;
        data_len = 0;
    }
    
	return ret;
}
/*********************************************************************
 * @fn      UART_Rx_Deal
 *
 * @brief   UART Rx data deal
 *
 * @return  none
 */
uint16_t uart_dma_len=0;
void UART_Rx_Deal(void)
{
    // u8 i, s;
    // u8 Data_add = 0;

    // if (Uart2_Rx() == Uart_Sync_Head1)
    // {
    //     if (Uart2_Rx() == Uart_Sync_Head2)
    //     {
    //         isp_cmd_t->Cmd = Uart2_Rx();
    //         Data_add += isp_cmd_t->Cmd;
    //         isp_cmd_t->Len = Uart2_Rx();
    //         Data_add += isp_cmd_t->Len;
    //         isp_cmd_t->Rev[0] = Uart2_Rx();
    //         Data_add += isp_cmd_t->Rev[0];
    //         isp_cmd_t->Rev[1] = Uart2_Rx();
    //         Data_add += isp_cmd_t->Rev[1];

    //         if ((isp_cmd_t->Cmd == CMD_IAP_PROM) || (isp_cmd_t->Cmd == CMD_IAP_VERIFY))
    //         {
    //             for (i = 0; i < isp_cmd_t->Len; i++) {
    //                 isp_cmd_t->data[i] = Uart2_Rx();
    //                 Data_add += isp_cmd_t->data[i];
    //             }
    //         }

    //         if (Uart2_Rx() == Data_add)
    //         {
    //             s = RecData_Deal();

    //             if (s != ERR_End)
    //             {
    //                 Uart2_SendData(0x00);
    //                 if (s == ERR_ERROR)
    //                 {
    //                     Uart2_SendData(0x01);
    //                 }
    //                 else
    //                 {
    //                     Uart2_SendData(0x00);
    //                 }
    //             }
    //         }
    //     }
    // }

    u8 s;
    while(dma_get_rx_lenth() != uart_dma_len){
        PRINT("0x%x \n\r",uart_dma_buf[uart_dma_len]);
		if(get_uart_command(uart_dma_buf[uart_dma_len])){
			if((isp_cmd_t->Cmd >= CMD_IAP_PROM) && (isp_cmd_t->Cmd <= CMD_IAP_END))
			{
                PRINT("cmd: 0x%x\n\r",isp_cmd_t->Cmd);
                log_data_hex((uint8_t*)isp_cmd_t->data,isp_cmd_t->Len);
				s = RecData_Deal();
                // PRINT("Ret:0x%x\r\n",s);
                if (s != ERR_End)
                {
                    Uart2_SendData(0x00);
                    if (s == ERR_ERROR)
                    {
                        Uart2_SendData(0x01);
                    }
                    else
                    {
                        Uart2_SendData(0x00);
                    }
                }
			}
		}
		uart_dma_len++;
		if(uart_dma_len >= DMA_BUFF_SIZE)uart_dma_len = 0;
	}
}


#endif


